#include "button.h"
#include "dialog.h"

using namespace std;

namespace Msp {
namespace GLtk {

Dialog::Dialog():
	stale(false)
{ }

void Dialog::add_button(Button &button, int code)
{
	add(button);
	button.signal_clicked.connect(sigc::bind(sigc::mem_fun(this, &Dialog::response), code));
}

void Dialog::button_release(int x, int y, unsigned button)
{
	Panel::button_release(x, y, button);
	if(stale)
		delete this;
}

bool Dialog::key_release(unsigned key, unsigned mod)
{
	bool result = Panel::key_release(key, mod);
	if(stale)
		delete this;
	return result;
}

void Dialog::response(int code)
{
	on_response(code);
	signal_response.emit(code);
	stale = true;
}


Dialog::Loader::Loader(Dialog &d, WidgetMap &wm):
	DerivedObjectLoader<Dialog, Panel::Loader>(d, wm)
{
	add("action_button", &Loader::action_button);
}

void Dialog::Loader::action_button(const string &n, int c)
{
	RefPtr<Button> btn = new Button();
	load_sub(*btn);
	obj.add_button(*btn.get(), c);
	last_widget = wdg_map[n] = btn.release();
}

} // namespace GLtk
} // namespace Msp
