#ifndef MSP_GLTK_ENTRY_H_
#define MSP_GLTK_ENTRY_H_

#include "container.h"
#include "text.h"
#include "widget.h"

namespace Msp {
namespace GLtk {

class VSlider;

/**
Text entry field.

Special parts:

  text    The current text of the widget.  Graphics are ignored.
  cursor  Indicates the current input position.  Fill_x is ignored.
  slider  A vertical slider for multiline entries.
*/
class Entry: virtual public Widget, private Container
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Entry, Widget::Loader>
	{
	public:
		Loader(Entry &);

	private:
		void multiline(bool);
	};

	sigc::signal<void> signal_enter;

private:
	Text text;
	bool multiline;
	unsigned edit_width;
	unsigned edit_height;
	unsigned edit_pos;
	unsigned first_row;
	unsigned visible_rows;
	const Part *text_part;
	VSlider *slider;
	bool got_key_press;

public:
	Entry(const std::string & = std::string());

	virtual const char *get_class() const { return "entry"; }

private:
	virtual void autosize_special(const Part &, Geometry &) const;

public:
	void set_text(const std::string &);
	const std::string &get_text() const { return text.get(); }

	/** Sets the minimum size of the editing area, in characters and rows.  This
	only affects autosizing. */
	void set_edit_size(unsigned w, unsigned h);

	void set_multiline(bool);
	bool is_multiline() const { return multiline; }

private:
	virtual void rebuild_special(const Part &);
	virtual void render_special(const Part &, GL::Renderer &) const;

public:
	virtual bool key_press(unsigned, unsigned);
	virtual bool character(wchar_t);
	virtual void focus_out();
private:
	virtual void on_geometry_change();
	virtual void on_style_change();

	void set_edit_position(unsigned);
	void check_view_range();
	void slider_value_changed(double);
};

} // namespace GLtk
} // namespace Msp

#endif

