#ifndef MSP_GLTK_ROOT_H_
#define MSP_GLTK_ROOT_H_

#include <sigc++/trackable.h>
#include <msp/gl/camera.h>
#include <msp/gl/program.h>
#include <msp/graphics/window.h>
#include <msp/input/keyboard.h>
#include <msp/input/mouse.h>
#include <msp/input/touchscreen.h>
#include <msp/time/timestamp.h>
#include "panel.h"

namespace Msp {
namespace GLtk {

class Label;

/**
A Root is a special type of Panel that covers the entire viewport and receives
input from keyboard and mouse.
*/
class Root: public Panel, public sigc::trackable
{
public:
	sigc::signal<std::string, int, int> signal_tooltip;

private:
	const Resources &resources;
	Input::Keyboard *keyboard;
	InputMethod *input_method;
	Input::Mouse *mouse;
	Input::Touchscreen *touchscreen;
	bool own_input;
	Label *lbl_tooltip;
	int pointer_x;
	int pointer_y;
	Time::TimeStamp tooltip_timeout;
	Widget *tooltip_target;
	Msp::GL::Camera camera;
	Msp::GL::Program *shprog;

public:
	/** Creates a Root widget for a window.  The geometry is set to match the
	window's size, and input devices are created automatically. */
	Root(const Resources &, Graphics::Window &);

	/** Creates a Root widget with custom input devices.  If window is not null,
	it is used to set the widget's initial geometry. */
	Root(const Resources &, Graphics::Window *, Input::Keyboard *, Input::Mouse *, Input::Touchscreen * = 0);
private:
	void init(Graphics::Window *);
public:
	virtual ~Root();

	virtual const char *get_class() const { return "root"; }

	const Resources &get_resources() const { return resources; }
	virtual unsigned get_width() const { return geom.w; }
	virtual unsigned get_height() const { return geom.h; }

	void tick();
	void render() const;

private:
	bool button_press_event(unsigned);
	bool button_release_event(unsigned);
	bool axis_motion_event(unsigned, float, float);
	bool touch_press_event(unsigned);
	bool touch_release_event(unsigned);
	bool touch_motion_event(unsigned, float, float);

	void get_pointer(int &, int &);
	void get_touch(unsigned, int &, int &);
	void update_camera();

	virtual void on_geometry_change();
	virtual void on_child_added(Widget &);
};

} // namespace GLtk
} // namespace Msp

#endif
